%% qtbug.m - a script to demonstrate some octave-gui problems

clear all;
chdir (fileparts(mfilename("fullpath")));

%% parameters
nR = 15000;                           % half the random points generated
delta = 0.12;                         % portfolio perturbation parameter

load QTBUG.mat;
%{
   Variables in the current scope:
   A   BF  BT  P   XX
   A (253 x 4) : 2014-12-31-normalized adjusted closing prices for 4
     four large exchange traded funds:
       1. iShares 20+ Year Treasury Bond (TLT)
       2. iShares Cohen & Steers REIT (ICF)
       3. Vanguard Total Intl Bd Idx ETF (BNDX)
       4. PowerShares DB US Dollar Index Bullish (UUP)
     These prices cover the 253 market days from 2013-12-31 to
     2014-12-31.
   BF & BT : (e, sig) points along the false and true efficient
     frontiers, respectively.
   P : 2014-12-31 closing portfolios for F154 and X:
     pF154 = P(:, 1);  pX = P(:, 2);
                 TLT      ICF      BNDX     UUP
     pF154 = [ 0.13350  0.15108  0.43942  0.27600 ]';
     pX    = [ 0.11950  0.13876  0.47062  0.27112 ]';
   XX = [eXX, sigXX]';  A point on the false efficient frontier
     satisfying eXX = eF154 and sigXX < sigF154. Regardless, F154 is a
     2014-efficient unattended investment portfolio in the four exchange
     traded funds, and no investment portfolio XX actually exists 
     with the (eXX, sigXX) coordinates, even though
        eXX = E * pX  and  sigXX = sqrt(pX' * C * pX),
     whatever that means.
%}

tic                                   % time this run

%% generate (e, sig) points
XR1 = delta * rand(4, nR);
XR2 = delta * rand(4, nR);
ZR1 = XR1 - ones(4, 1) * mean(XR1);   % deviations from the mean
ZR2 = XR2 - ones(4, 1) * mean(XR2);   % deviations from the mean
PR1 = P(:, 1) * ones(1, nR) + ZR1;    % random portfolios near P(:, 1)
PR2 = P(:, 2) * ones(1, nR) + ZR2;    % random portfolios near P(:, 2)
AR = A * [PR1, PR2];                  % 2 * nR random price vectors
RR = 100 * (AR([2 : 253], :) ./ AR([1 : 252], :) - 1);  % daily returns
ER = sum(RR);                         % expected returns
SigR = sqrt(252) * std(RR, 1);        % stdevs of return

%% special (e, sig) points
pF154 = P(:, 1);
aF154 = A * pF154;
rF154 = 100 * (aF154(2 : 253) ./ aF154(1 : 252) - 1);
eF154 = sum(rF154);                   % eF154   = 14.0185 = eXX below
sigF154 = sqrt(252) * std(rF154, 1);  % sigF154 =  2.7737
pX = P(:, 2);
aX = A * pX;
rX = 100 * (aX(2 : 253) ./ aX(1 : 252) - 1);
eX = sum(rX);                         % eX      = 13.5429
sigX = sqrt(252) * std(rX, 1);        % sigX    =  2.6496
eXX = XX(1);                          % eXX     = 14.0185
sigXX = XX(2);                        % sigXX    = 2.7052

%% for a MATLAB computation of the false efficient frontier
%{
R = 100 * (A([2 : 253], :) ./ A([1 : 252], :) - 1);
E = sum(R);                           % component fund 'AssetMean'
C = 252 * cov(R, 1);                  % component fund 'AssetCovar'
%{
  E = [  24.7131   29.8709    8.4061   10.9336 ];
  C = [
         112.2951   -8.0438   13.9776  -15.7922
          -8.0438  128.6672    1.5086    1.8758
          13.9776    1.5086    4.9623    2.6484
         -15.7922    1.8758   -2.6484   29.8435
      ];
%}
%}

% obtainable/unobtainable patch
EF = [BF(1, :), 14.1 , 13.5];
SigF = [BF(2, :), 2.9, 2.9];

% obtainable patch
ET = [BT(1, :), 14.1 , 13.5];
SigT = [BT(2, :), 2.9, 2.9];

% parameter text
nRtxt = sprintf("nR    = %d", nR);
deltatxt = sprintf("delta =  %.3f", delta);

close all;
hold on;

patch(EF, SigF, [0.925, 0.886, 0.843], "edgecolor", "none");
patch(ET, SigT, [0.85, 0.85, 1], "edgecolor", "none");

rectx = [13.5, 14.1, 14.1, 13.5, 13.5];
recty = [2.6, 2.6, 2.9, 2.9, 2.6];
plot( ...
  ER, SigR, ".b", "markersize", 2, ...
  rectx, recty, "-k", ...
  EF, SigF, "-k", ...
  eF154, sigF154, ".r", "markersize", 12, "markeredgecolor", "k", ...
  eX, sigX, ".y", "markersize", 12, "markeredgecolor", "k", ...
  eXX, sigXX, ".k", "markersize", 10 ...
);
axis([13.5, 14.1, 2.6, 2.9], "equal");
text(14.002, 2.758, "F154", "fontsize", 12, "fontweight", "bold");
text(14.009, 2.690, "XX", "fontsize", 12, "fontweight", "bold");
text(13.538, 2.635, "X", "fontsize", 12, "fontweight", "bold");
text(13.79, 2.636, "false efficient frontier", "fontsize", 12, ...
  "rotation", 14);
text(13.74, 2.660, "unobtainable (e, sig)", "fontsize", 12, ...
  "rotation", 14);
text(13.70, 2.677, "true efficient frontier", "fontsize", 12, ...
  "rotation", 15);
text(13.56, 2.83, "obtainable (e, sig)", "fontsize", 12, ...
  "rotation", 14);
text(13.99, 2.64, nRtxt, "fontsize", 12);
text(13.99, 2.62, deltatxt, "fontsize", 12);
set(get(gca, "xlabel"), "string", ...
  "annualized mean daily return", "fontsize", 12);
set(get(gca, "ylabel"), "string", ...
  "annualized standard deviation of daily return", "fontsize", 12);
set(gca, "tickdir", "out", ...
  "xtick", [13.5, 13.6, 13.7, 13.8, 13.9, 14.0, 14.1], ...
  "ytick", [2.6, 2.7, 2.8, 2.9]);
set(gca, "title", "New Markowitz: how not to do mean-variance");
set(get(gca, "title"), "fontsize", 12);

hold off;

toc                                   % elapsed time
